import torch
import torch.nn.functional as F

def classify_claim(claim: str, context: str, model, tokenizer, device: torch.device):
    """
    Classifies the given claim based on the provided context using a pre-trained model.
    
    Args:
        claim (str): The claim that needs to be verified.
        context (str): The supporting or opposing context for the claim.
        model: The pre-trained classification model.
        tokenizer: The tokenizer corresponding to the model.
        device (torch.device): The device (CPU or GPU) on which to run the model.
    
    Returns:
        tuple: A tuple containing:
            - prob (torch.Tensor): The probability of the predicted class.
            - pred (int): The predicted class label.
    """
    
    model.to(device)
    model.eval()

    encoding = tokenizer(
        claim,
        context,
        truncation="only_second",
        add_special_tokens=True,
        max_length=256,
        padding='max_length',
        return_attention_mask=True,
        return_token_type_ids=False,
        return_tensors='pt',
    )

    input_ids = encoding['input_ids'].to(device)
    attention_mask = encoding['attention_mask'].to(device)

    with torch.no_grad():
        outputs = model(input_ids=input_ids, attention_mask=attention_mask)

    probabilities = F.softmax(outputs["logits"], dim=1)
    prob, pred = torch.max(probabilities, dim=1)

    return prob, pred.item()



import numpy as np
import torch
import torch.nn as nn
from pyvi import ViTokenizer
from rank_bm25 import BM25Okapi
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics.pairwise import cosine_similarity
from sklearn.preprocessing import MinMaxScaler

from ..data_processing.pipline import split_sentence, preprocess_text


def extract_evidence_qatc(claim, context, model_evidence_QA, tokenizer_QA, device):
    """
    Extracts evidence from the given context using a QA model.
    
    Args:
        claim (str): The claim statement.
        context (str): The context containing possible evidence.
        model_evidence_QA (torch.nn.Module): The QA model used for evidence extraction.
        tokenizer_QA: Tokenizer corresponding to the QA model.
        device (str): The device (CPU/GPU) to run the model on.

    Returns:
        str or int: The extracted evidence sentence or -1 if no valid evidence is found.
    """
    model_evidence_QA.to(device).eval()
    inputs = tokenizer_QA(claim, context, return_tensors="pt", truncation=True, max_length=512, padding="max_length")
    inputs = {key: value.to(device) for key, value in inputs.items()}

    with torch.no_grad():
        outputs = model_evidence_QA(**inputs)

    start_index, end_index = np.argmax(outputs.start_logits.cpu().numpy()), np.argmax(outputs.end_logits.cpu().numpy())
    evidence = tokenizer_QA.decode(inputs['input_ids'][0][start_index:end_index + 1]).replace('<s>', '').replace('</s>', '')

    if not evidence or len(split_sentence(evidence)) > 1:
        return -1

    for line in split_sentence(context):
        if preprocess_text(evidence) in preprocess_text(line):
            return line

    print('Error: Evidence not found in context')
    return evidence


def extract_evidence_qatc_faster(claim, context, full_context, model_evidence_QA, tokenizer_QA, device):
    """
    Faster evidence extraction method using batch processing.

    Args:
        claim (str or list): The claim(s) to be processed.
        context (str or list): The corresponding context(s).
        full_context (str): The full original context.
        model_evidence_QA (torch.nn.Module): The QA model used for evidence extraction.
        tokenizer_QA: Tokenizer corresponding to the QA model.
        device (str): The device (CPU/GPU) to run the model on.

    Returns:
        str or int: The extracted evidence or -1 if no valid evidence is found.
    """
    claim, context = ([claim], [context]) if isinstance(claim, str) else (claim, context)
    
    model_evidence_QA.to(device).eval()
    inputs = tokenizer_QA(claim, context, return_tensors="pt", truncation=True, max_length=512, padding="max_length")
    inputs = {key: value.to(device) for key, value in inputs.items()}
    
    with torch.no_grad():
        outputs = model_evidence_QA(**inputs)

    start_indices = outputs.start_logits.argmax(dim=-1).tolist()
    end_indices = outputs.end_logits.argmax(dim=-1).tolist()

    evidences = tokenizer_QA.batch_decode(
        [inputs['input_ids'][0][s:e + 1] for s, e in zip(start_indices, end_indices)],
        skip_special_tokens=True
    )

    valid_evidence = [(preprocess_text(evi.lstrip(".")), i) for i, evi in enumerate(evidences) if len(preprocess_text(evi).split()) > 3]

    if len(valid_evidence) != 1:
        return -1  

    evidence_text = valid_evidence[0][0]
    for line in split_sentence(full_context):
        if evidence_text in preprocess_text(line):
            return line  

    return -1  


def tfidf_topk(claim, context, threshold=0.6, top_k=1):
    """
    Retrieves the top-k relevant sentences from the context using TF-IDF similarity.

    Args:
        claim (str): The claim statement.
        context (str): The context containing multiple sentences.
        threshold (float): Threshold for sentence length ratio.
        top_k (int): Number of top sentences to return.

    Returns:
        list: List of tuples (similarity_score, sentence).
    """
    corpus = split_sentence(context)
    processed_claim = preprocess_text(ViTokenizer.tokenize(claim).lower())
    
    corpus_processed = []
    for i, sentence in enumerate(corpus):
        sentence = preprocess_text(ViTokenizer.tokenize(sentence).lower())
        if i > 0 and 1 < len(sentence.split()) / len(processed_claim.split()) < threshold:
            sentence = f"{corpus[i-1]}. {sentence}"
        corpus_processed.append(sentence)

    vectorizer = TfidfVectorizer()
    tfidf_matrix = vectorizer.fit_transform(corpus_processed + [processed_claim])
    cosine_sim = cosine_similarity(tfidf_matrix[-1], tfidf_matrix[:-1]).flatten()

    return sorted(zip(cosine_sim, corpus), reverse=True)[:top_k]


def bm25_topk(claim, context, top_k=None):
    """
    Retrieves the top-k relevant sentences from the context using BM25 ranking.

    Args:
        claim (str): The claim statement.
        context (str): The context containing multiple sentences.
        top_k (int, optional): Number of top sentences to return. If None, returns all.

    Returns:
        list: List of tuples (score, sentence).
    """
    sentences = split_sentence(context)
    tokenized_context = [sentence.split(' ') for sentence in sentences]
    bm25 = BM25Okapi(tokenized_context)
    scores = bm25.get_scores(claim.split())

    normalized_scores = MinMaxScaler().fit_transform(np.array(scores).reshape(-1, 1)).flatten()
    sorted_sentences = sorted(zip(normalized_scores, sentences), reverse=True)

    return sorted_sentences[:top_k] if top_k else sorted_sentences


def mean_pooling(model_output, attention_mask):
    """
    Computes mean pooling of token embeddings for sentence representation.

    Args:
        model_output (torch.Tensor): Output embeddings from a transformer model.
        attention_mask (torch.Tensor): Attention mask.

    Returns:
        torch.Tensor: Mean-pooled sentence representation.
    """
    token_embeddings = model_output[0]
    mask_expanded = attention_mask.unsqueeze(-1).expand(token_embeddings.size()).float()
    return torch.sum(token_embeddings * mask_expanded, dim=1) / torch.clamp(mask_expanded.sum(dim=1), min=1e-9)


def extract_evidence_tfidf_qatc(claim, context, model_evidence_QA, tokenizer_QA, device, confidence_threshold=0.5, length_ratio_threshold=0.6, is_qatc_faster=False):
    """
    Finds the most relevant evidence for a given claim in the context.

    Args:
        claim (str): The claim statement.
        context (str): The context containing possible evidence.
        model_evidence_QA (torch.nn.Module): The QA model used for evidence extraction.
        tokenizer_QA: Tokenizer corresponding to the QA model.
        device (str): The device (CPU/GPU) to run the model on.
        confidence_threshold (float): Similarity threshold for TF-IDF selection.
        is_qatc_faster (bool): Whether to use the faster QATC method.

    Returns:
        str: The best-matching evidence sentence.
    """
    evidence_tf = tfidf_topk(claim, context, top_k=1, threshold=length_ratio_threshold)[0]
    if evidence_tf[0] > confidence_threshold:
        return evidence_tf[1]

    sentences = split_sentence(context)
    if len(context.split()) <= 400:  
        evidence = extract_evidence_qatc(claim, context, model_evidence_QA, tokenizer_QA, device)
        return evidence if evidence != -1 else evidence_tf[1]

    tokens = context.split(' ')
    token_sentences = [l.split(' ') for l in sentences]

    tmp_context_token = []
    tmp_context = []
    sub_contexts = []
    for idx in range(len(sentences)):
        check = True
        if len(token_sentences[idx] + tmp_context_token) <=400:
            tmp_context_token += token_sentences[idx]
            tmp_context.append(sentences[idx])
            check = False
        
        if len(token_sentences[idx] + tmp_context_token) > 400 or idx == len(sentences) - 1:
            context_sub = '. '.join(tmp_context)
            if len(context_sub)== 0: 
                continue
            sub_contexts.append(context_sub)

            if check:
                tmp_context_token = token_sentences[idx]
                tmp_context = [sentences[idx]]
            else:
                tmp_context_token = []
                tmp_context = []

    if is_qatc_faster:
        return extract_evidence_qatc_faster(claim, sub_contexts, context, model_evidence_QA, tokenizer_QA, device)

    for sub_context in sub_contexts:
        evidence = extract_evidence_qatc(claim, sub_context, model_evidence_QA, tokenizer_QA, device)
        if evidence != -1:
            return evidence

    return evidence_tf[1]




import torch
from torch.utils.data import Dataset
class Data(Dataset):
    def __init__(self, df, tokenizer, config, max_len=256):
        self.df = df
        self.max_len = max_len
        self.tokenizer = tokenizer
        self.config = config
    
    def __len__(self):
        return len(self.df)
    def __getitem__(self, index):
        row = self.df.iloc[index]
        claim, context, label, ids = self.get_input_data(row)
        
        encoding = self.tokenizer.encode_plus(
            claim,
            context,
            truncation=True,
            add_special_tokens=True,
            max_length=self.max_len,
            padding='max_length',
            return_attention_mask=True,
            return_token_type_ids=False,
            return_tensors='pt', 
        )
        
        return {
            'id': ids,
            'input_ids': encoding['input_ids'].flatten(),
            'attention_masks': encoding['attention_mask'].flatten(),
            'targets': torch.tensor(label, dtype=torch.long),
        }


    def labelencoder(self, text):
        label_map = {
            3: {'NEI': 0, 'SUPPORTED': 1, 'REFUTED': 2},
            2: {'SUPPORTED': 0, 'REFUTED': 1} 
        }
        return label_map[self.config.n_classes].get(text, 1)

    def get_input_data(self, row):
        claim = row['claim']
        context = row['context']
        ids = row['id']
        label = self.labelencoder(row['verdict'])
        
        return str(claim), str(context), label, ids


from omegaconf import OmegaConf
import json

def write_json(path, data):
    with open(path, 'w', encoding='utf-8') as file:
        json.dump(data, file, indent=4, ensure_ascii=False)  
        
def load_config(file_path):
    """
    Load a YAML configuration file using OmegaConf.
    """
    config = OmegaConf.load(file_path)
    return config


import pandas as pd
from datasets import Dataset

def preprocess_training_dataset(examples, tokenizer, config):
    questions = [q.strip() for q in examples["claim"]]
    inputs = tokenizer(
        questions,
        examples["context"],
        max_length=512,
        truncation="only_second",
        return_offsets_mapping=True,
        padding="max_length",
    )

    offset_mapping = inputs.pop("offset_mapping")
    start_positions = []
    end_positions = []

    for i, offset in enumerate(offset_mapping):
        if pd.notna(examples["evidence"][i]) and examples["evidence"][i] not in ["", " ","nan","none"]:
    #         answer = answers[i]
            start_char =  examples["context"][i].find(examples["evidence"][i])
            if start_char == -1:
                print("Evidence not found in context")
            end_char = start_char + len(examples["evidence"][i])
            sequence_ids = inputs.sequence_ids(i)

            # Find the start and end of the context
            idx = 0
            try:
                while sequence_ids[idx] != 1:
                    idx += 1
            except:
                print(questions[i])
                print("+++++")
                print(examples["context"][i])
                print(sequence_ids)
            context_start = idx
            while sequence_ids[idx] == 1:
                idx += 1
            context_end = idx - 1

            # If the answer is not fully inside the context, label it (0, 0)
            if offset[context_start][0] > end_char or offset[context_end][1] < start_char:
                start_positions.append(0)
                end_positions.append(0)
            else:
                # Otherwise it's the start and end token positions
                idx = context_start
                while idx <= context_end and offset[idx][0] <= start_char:
                    idx += 1
                start_positions.append(idx - 1)

                idx = context_end
                while idx >= context_start and offset[idx][1] >= end_char:
                    idx -= 1
                end_positions.append(idx + 1)
        else:
            start_positions.append(0)
            end_positions.append(0)

    inputs["start_positions"] = start_positions
    inputs["end_positions"] = end_positions
    inputs["id"] = examples["id"]
    list_tagg_rational = []
    for i in range(len(end_positions)):
        tagging = [0]*len(inputs['input_ids'][i])
        if end_positions[i] != start_positions[i]:
            tagging[start_positions[i]:end_positions[i] + 1] = [1] * (end_positions[i] - start_positions[i] + 1)
        list_tagg_rational.append(tagging) 
    inputs['Tagging'] = list_tagg_rational  
    return inputs


def load_data(config, tokenizer):   
    train_dataset = None
    test_dataset = None
    
    if config.is_train:
        train = pd.read_csv(config.train_data)

        train.id = range(len(train))
        train = train.astype(str)

        train = Dataset.from_dict({
            "context": train.context,
            "claim": train['claim'],
            "verdict": train.verdict,
            "evidence": train.evidence,
            "id": train.id,
        })

        train_dataset = train.map(
            lambda examples: preprocess_training_dataset(examples, tokenizer,config),
            batched=True,
            remove_columns=train.column_names,
        )
        
    if config.is_eval:
        test = pd.read_csv(config.eval_data)

        test.id = range(len(test))
        test = test.astype(str)

        test = Dataset.from_dict({
            "context": test.context,
            "claim": test.claim,
            "verdict": test.verdict,
            "evidence": test.evidence,
            "id": test.id,
        })

        test_dataset = test.map(
            lambda examples: preprocess_training_dataset(examples, tokenizer,config),
            batched=True,
            remove_columns=test.column_names,
        )
        
    if config.is_train and config.is_eval:
        return train_dataset, test_dataset
    elif config.is_train:
        return train_dataset
    elif config.is_eval:
        return test_dataset
    else:
        return None


from transformers import PreTrainedModel, AutoModel, PretrainedConfig
import torch
import torch.nn as nn
from .loss import FocalLoss

class ClaimModelConfig(PretrainedConfig):
    model_type = "claim_verification"

    def __init__(
        self,
        model_name=None,
        num_labels=3,
        dropout=0.3,
        loss_type='ce',
        **kwargs
    ):
        super().__init__(**kwargs)
        self.model_name = model_name
        self.num_labels = num_labels
        self.dropout = dropout
        self.loss_type = loss_type

class ClaimModelForClassification(PreTrainedModel):
    config_class = ClaimModelConfig
    base_model_prefix = "claim_verification"

    def __init__(self, config):
        super().__init__(config)
        self.config = config
        self.bert = AutoModel.from_pretrained(config.model_name)
        self.dropout = nn.Dropout(p=config.dropout)
        self.fc = nn.Linear(self.bert.config.hidden_size, config.num_labels)

        if self.config.loss_type == 'focal':
            self.loss_fn = FocalLoss()
        else:
            self.loss_fn = nn.CrossEntropyLoss()

        self.post_init()

    def forward(self, 
                input_ids, 
                attention_mask=None,
                token_type_ids=None,
                position_ids=None,
                head_mask=None,
                inputs_embeds=None, 
                labels=None,
                **kwargs):
        outputs = self.bert(
            input_ids=input_ids,
            attention_mask=attention_mask,
            # token_type_ids=token_type_ids,
            position_ids=position_ids,
            head_mask=head_mask,
            inputs_embeds=inputs_embeds,
            **kwargs
        )

        cls_output = outputs[1]  
        x = self.dropout(cls_output)
        logits = self.fc(x)
        
        if labels is not None:
            loss = self.loss_fn(logits, labels)
            return {"loss": loss, "logits": logits}  
        
        return {"logits": logits}



from transformers import PreTrainedModel, AutoModel, PretrainedConfig, RobertaModel, XLMRobertaModel
from transformers.modeling_outputs import QuestionAnsweringModelOutput
import torch
import torch.nn.functional as F
import torch.nn as nn
from .loss import comboLoss  

class QATCConfig(PretrainedConfig):
    model_type = "qatc"

    def __init__(
        self, 
        model_name=None,
        freeze_text_encoder=False,
        alpha=1.0,
        beta=0.01,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.model_name = model_name
        self.freeze_text_encoder = freeze_text_encoder
        self.alpha = alpha  
        self.beta = beta    

class Rational_Tagging(nn.Module):
    def __init__(self, hidden_size):
        super(Rational_Tagging, self).__init__()
        self.W1 = nn.Linear(hidden_size, hidden_size)
        self.w2 = nn.Linear(hidden_size, 1)

    def forward(self, h_t):
        h_1 = self.W1(h_t)
        h_1 = F.relu(h_1)
        p = self.w2(h_1)
        p = torch.sigmoid(p)  
        return p

class QATCForQuestionAnswering(PreTrainedModel):
    config_class = QATCConfig
    base_model_prefix = "qa_model"

    def __init__(self, config):
        super(QATCForQuestionAnswering, self).__init__(config)
        self.config = config
        if "deberta" in self.config.model_name:
            self.model = AutoModel.from_pretrained(self.config.model_name) 
        elif "info" in self.config.model_name:
            self.model = XLMRobertaModel.from_pretrained(self.config.model_name)
        else:
            self.model = RobertaModel.from_pretrained(self.config.model_name)

        if getattr(self.config, "freeze_text_encoder", False):
            print("Freezing text encoder weights")
            for param in self.model.parameters():
                param.requires_grad = False

        self.qa_outputs = nn.Sequential(
            nn.Linear(self.model.config.hidden_size, self.model.config.hidden_size),
            nn.LayerNorm(self.model.config.hidden_size),
            nn.ReLU(),
            nn.Dropout(0.1),
            nn.Linear(self.model.config.hidden_size, 2),
        )

        self.tagging = Rational_Tagging(self.model.config.hidden_size)
        self.loss_fn = comboLoss(config)
        self.init_weights()
        self.model.pooler = None

    def forward(
        self,
        input_ids,
        attention_mask=None,
        token_type_ids=None,
        position_ids=None,
        head_mask=None,
        inputs_embeds=None,
        output_attentions=None,
        output_hidden_states=None,
        return_dict=None,
        start_positions=None,
        end_positions=None,
        tagging_labels=None,
        **kwargs
    ):
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        outputs = self.model(
            input_ids=input_ids,
            attention_mask=attention_mask,
            token_type_ids=token_type_ids,
            position_ids=position_ids,
            head_mask=head_mask,
            inputs_embeds=inputs_embeds,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
            **kwargs
        )
        sequence_output = outputs[0]  

        rational_tag_logits = self.tagging(sequence_output)  

        logits = self.qa_outputs(sequence_output)  
        start_logits, end_logits = logits.split(1, dim=-1)
        start_logits = start_logits.squeeze(-1).contiguous()
        end_logits = end_logits.squeeze(-1).contiguous()

        loss_inputs = {
            "attention_mask": attention_mask,
            "start_logits": start_logits,
            "end_logits": end_logits,
            "start_positions": start_positions,
            "end_positions": end_positions,
            "Tagging": tagging_labels,
            "pt": rational_tag_logits
        }

        if start_logits == None and end_logits == None and tagging_labels == None:
            total_loss = self.loss_fn(loss_inputs)
        else:
            total_loss = None

        if not return_dict:
            return (total_loss, start_logits, end_logits, rational_tag_logits) + outputs[2:]

        return QuestionAnsweringModelOutput(
            loss=total_loss,
            start_logits=start_logits,
            end_logits=end_logits,
            hidden_states=outputs.hidden_states,
            attentions=outputs.attentions
        )



import torch
import torch.nn as nn
import torch.nn.functional as F

class FocalLoss(nn.Module):
    def __init__(self, gamma=2.0, alpha=0.25, reduction='mean'):
        """
        Focal Loss to replace CrossEntropyLoss.
        Args:
            gamma (float): Adjusts the focus on hard examples (default = 2.0).
            alpha (float): Weight for the positive class to balance (default = 0.25).
            reduction (str): 'mean' or 'sum'.
        """
        super(FocalLoss, self).__init__()
        self.gamma = gamma
        self.alpha = alpha
        self.reduction = reduction

    def forward(self, logits, targets):
        """
        Args:
            logits: (batch_size, num_classes) - Output of the model.
            targets: (batch_size) - Ground truth labels.

        Returns:
            loss: Focal loss value
        """
        ce_loss = F.cross_entropy(logits, targets, reduction='none')
        pt = torch.exp(-ce_loss)
        focal_loss = self.alpha * (1 - pt) ** self.gamma * ce_loss

        if self.reduction == 'mean':
            return focal_loss.mean()
        elif self.reduction == 'sum':
            return focal_loss.sum()
        else:
            return focal_loss


import torch
import pandas as pd
from torch.utils.data import DataLoader
from transformers import AutoTokenizer, XLMRobertaTokenizerFast
from torch.optim import AdamW
from transformers import get_linear_schedule_with_warmup
import torch.nn as nn
import torch.nn.functional as F
from sklearn.metrics import f1_score, accuracy_score, classification_report, precision_score, recall_score, cohen_kappa_score, matthews_corrcoef, confusion_matrix, roc_auc_score
from sklearn.utils.class_weight import compute_class_weight
import numpy as np
from tqdm import tqdm
import os
import time
import multiprocessing

from data_utils import Data
from model import ClaimVerificationConfig, ClaimVerificationModel

multiprocessing.set_start_method('spawn', force=True)

def set_seed(seed):
    torch.manual_seed(seed)
    if torch.cuda.is_available():
        torch.cuda.manual_seed_all(seed)
    
    np.random.seed(seed)
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.benchmark = False

def count_parameters(model):
    print(f'The model has {sum(p.numel() for p in model.parameters() if p.requires_grad):,} trainable parameters')
    print(f'The model has {sum(p.numel() for p in model.parameters()):,} parameters')

def main(args):
    device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

    train_data = pd.read_csv(args.train_data).reset_index()
    dev_data = pd.read_csv(args.dev_data).reset_index()

    train_data["id"] = train_data.index
    dev_data["id"] = dev_data.index

    if args.n_classes == 2:
        train_data = train_data[train_data["verdict"] != "NEI"]
        dev_data = dev_data[dev_data["verdict"] != "NEI"]

    output_dir = args.output_dir
    os.makedirs(output_dir, exist_ok=True)
 
    tokenizer = AutoTokenizer.from_pretrained(args.model_name)

    config = ClaimVerificationConfig(
        model_name=args.model_name,
        num_labels=args.n_classes,
        dropout=args.dropout_prob,
        loss_type=args.type_loss
    )
    model = ClaimVerificationModel(config).to(device)
    count_parameters(model)

    train_dataset = Data(train_data, tokenizer, args, max_len=args.max_len)
    dev_dataset = Data(dev_data, tokenizer, args, max_len=args.max_len)

    train_loader = DataLoader(train_dataset, batch_size=args.batch_size, shuffle=True, num_workers=args.num_workers)
    dev_loader = DataLoader(dev_dataset, batch_size=args.batch_size, shuffle=True, num_workers=args.num_workers)

    optimizer = AdamW(model.parameters(), lr=args.lr)

    lr_scheduler = get_linear_schedule_with_warmup(
                optimizer, 
                num_warmup_steps=0, 
                num_training_steps=len(train_loader)*args.epochs
            )

    info_epoch = {}
    logs = []
    best_acc = 0
    cnt = 0
    i = 0

    total_time = time.time()
    for epoch in range(args.epochs):
        set_seed(42)
        info_epoch[epoch] = {}
        print(f'Epoch {epoch+1}/{args.epochs}')
        print('-'*30)
        start_training_time = time.time()

        model.train()
        train_losses = []
        true_labels = []
        predicted_labels = []

        for data in tqdm(train_loader):
            y_true = data['targets'].to(device)
            input_ids = data['input_ids'].to(device)
            attention_mask = data['attention_masks'].to(device)

            outputs = model(input_ids=input_ids, attention_mask=attention_mask, labels=y_true)
            loss = outputs["loss"]
            logits = outputs["logits"] 
            train_losses.append(loss.item())

            loss.backward()
            if (i + 1) % args.accumulation_steps == 0:
                optimizer.step()
                lr_scheduler.step() 
                optimizer.zero_grad()
                nn.utils.clip_grad_norm_(model.parameters(), max_norm=1.0)
            i += 1

            _, pred = torch.max(logits, dim=1)
            true_labels.extend(y_true.cpu().numpy())
            predicted_labels.extend(pred.cpu().numpy())

        train_f1 = f1_score(true_labels, predicted_labels, average='macro')
        train_acc = accuracy_score(true_labels, predicted_labels)
        epoch_training_time = time.time() - start_training_time
        print(f'Training time: {epoch_training_time:.2f}s Train Loss: {np.mean(train_losses):.4f} F1: {train_f1:.4f} Acc: {train_acc:.4f}')

        model.eval()
        eval_losses = []
        y_true_list = []
        y_pred_list = []

        start_eval_time = time.time()
        with torch.no_grad():
            for data in tqdm(dev_loader):
                y_true = data['targets'].to(device)
                input_ids = data['input_ids'].to(device)
                attention_mask = data['attention_masks'].to(device)

                outputs = model(input_ids=input_ids, attention_mask=attention_mask, labels=y_true)
                loss = outputs["loss"]
                logits = outputs["logits"]
                eval_losses.append(loss.item())

                _, pred = torch.max(logits, dim=1)
                y_true_list.extend(y_true.cpu().numpy())
                y_pred_list.extend(pred.cpu().numpy())

        dev_f1 = f1_score(y_true_list, y_pred_list, average='macro')
        dev_acc = accuracy_score(y_true_list, y_pred_list)
        epoch_eval_time = time.time() - start_eval_time

        print(f'Dev time: {epoch_eval_time}s Dev Loss: {np.mean(eval_losses):.4f} F1: {dev_f1:.4f} Acc: {dev_acc:.4f}')

        info_epoch[epoch] = {
            "time_train": epoch_training_time,
            "epoch": epoch,
            "train_loss": np.mean(train_losses),
            "train_acc": train_acc,
            "f1-train": train_f1,
            "time_val": epoch_eval_time,
            "val_acc": dev_acc,
            "val_loss": np.mean(eval_losses),
            "f1-val": dev_f1
        }

        if dev_f1 > best_acc:
            cnt = 0
            tokenizer.save_pretrained(output_dir)
            model.save_pretrained(output_dir)
            config.save_pretrained(output_dir)
            print(f'Saved best_model at epoch {epoch+1}')
            best_acc = dev_f1
        else:
            cnt += 1

        if cnt >= args.patience:
            print('Early stopping')
            break

        torch.cuda.empty_cache()

    print(f'Total training time: {time.time() - total_time:.2f}s')
    pd.DataFrame(info_epoch).T.to_csv(os.path.join(output_dir, 'info_epoch.csv'), index=False)

def parse_args():
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('--train_data', type=str, default='data/train.csv')
    parser.add_argument('--dev_data', type=str, default='data/dev.csv')
    parser.add_argument('--model_name', type=str, default='MoritzLaurer/ernie-m-large-mnli-xnli')
    parser.add_argument('--lr', type=float, default=1e-5)
    parser.add_argument('--epochs', type=int, default=10)
    parser.add_argument('--batch_size', type=int, default=8)
    parser.add_argument('--max_len', type=int, default=256)
    parser.add_argument('--num_workers', type=int, default=2)
    parser.add_argument('--patience', type=int, default=5)
    parser.add_argument('--type_loss', type=str, default='cross_entropy')
    parser.add_argument('--output_dir', type=str, default='output')
    parser.add_argument('--n_classes', type=int, default=3)
    parser.add_argument('--is_weighted', type=int, default=0)
    parser.add_argument('--dropout_prob', type=float, default=0.3)
    parser.add_argument('--accumulation_steps', type=int, default=1)
    return parser.parse_args()

if __name__ == '__main__':
    args = parse_args()
    main(args)



from transformers import AutoTokenizer
from tqdm.notebook import tqdm
import torch
import pandas as pd
import json
import argparse

from .data_processing.pipline import process_data, load_data
from .ser.ser_eval import extract_evidence_tfidf_qatc
from .ser.qatc_model import QATCForQuestionAnswering
from .tvc.tvc_eval import classify_claim
from .tvc.model import ClaimModelForClassification

class SemViQAPipeline:
    def __init__(self, model_evidence_QA, model_bc, model_tc, thres_evidence=0.5, length_ratio_threshold=0.6, is_qatc_faster=False, device=None):
        self.device = device if device else torch.device("cuda" if torch.cuda.is_available() else "cpu")
        self.thres_evidence = thres_evidence
        self.length_ratio_threshold = length_ratio_threshold
        self.is_qatc_faster = is_qatc_faster
        
        # Load models
        self.tokenizer_QA = AutoTokenizer.from_pretrained(model_evidence_QA)
        self.model_evidence_QA = QATCForQuestionAnswering.from_pretrained(model_evidence_QA)
        
        self.tokenizer_classify = AutoTokenizer.from_pretrained(model_tc)
        self.model_tc = ClaimModelForClassification.from_pretrained(model_tc)
        self.model_bc = ClaimModelForClassification.from_pretrained(model_bc, num_labels=2)
        
    def predict(self, claim, context, return_evidence_only=False):
        evidence = extract_evidence_tfidf_qatc(
            claim, context, self.model_evidence_QA, self.tokenizer_QA, self.device, confidence_threshold=self.thres_evidence, length_ratio_threshold=self.length_ratio_threshold, is_qatc_faster=self.is_qatc_faster
        )
        
        if return_evidence_only:
            return {"evidence": evidence}
        
        verdict = "NEI"
        prob3class, pred_tc = classify_claim(claim, evidence, self.model_tc, self.tokenizer_classify, self.device)
        
        if pred_tc != 0:
            prob2class, pred_bc = classify_claim(claim, evidence, self.model_bc, self.tokenizer_classify, self.device)
            verdict = "SUPPORTED" if pred_bc == 0 else "REFUTED" if prob2class > prob3class else ["NEI", "SUPPORTED", "REFUTED"][pred_tc]
        
        return {"verdict": verdict, "evidence": evidence}

    def process_batch(self, data_path, output_path, return_evidence_only=False):
        data = pd.read_csv(data_path) if "csv" in data_path else pd.read_json(data_path).T
        print(f'Load data: {len(data)} samples')
        
        data["id"] = data.index
        test_data = load_data(data)
        
        results = {}
        for idx, item in tqdm(test_data.items()):
            results[str(idx)] = self.predict(item[0]['claim'], item[0]['context'], return_evidence_only)
        
        with open(output_path, "w", encoding="utf-8") as json_file:
            json.dump(results, json_file, ensure_ascii=False, indent=4)
        
        print(pd.DataFrame(results).T.verdict.value_counts() if not return_evidence_only else "Evidence extraction completed.")

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--data_path", type=str, default="data/test.json", help="Path to data")
    parser.add_argument("--output_path", type=str, default="output.json", help="Path to output")
    parser.add_argument("--model_evidence_QA", type=str, default="QATC", help="Model evidence QA")  
    parser.add_argument("--model_bc", type=str, default="bc", help="Model 2 class") 
    parser.add_argument("--model_tc", type=str, default="tc", help="Model 3 class") 
    parser.add_argument("--thres_evidence", type=float, default=0.5, help="Threshold evidence")
    parser.add_argument("--return_evidence_only", action="store_true", help="Only extract evidence without classification")
    args = parser.parse_args()

    semviqa = SemViQAPipeline(args.model_evidence_QA, args.model_bc, args.model_tc, args.thres_evidence)
    semviqa.process_batch(args.data_path, args.output_path, args.return_evidence_only)



from setuptools import setup, find_packages

setup(
    name="semviqa",
    version="1.1.7",
    packages=find_packages(),
    install_requires=[
        "transformers==4.42.3",
        "datasets==2.20.0",
        "pandas==2.2.2",
        "numpy==1.26.4",
        "underthesea==6.8.4",
        "gc-python-utils==0.0.1",
        "tqdm==4.66.4",
        "safetensors==0.4.3",
        "sentence-transformers==3.0.1",
        "scikit-learn==1.2.2",
        "matplotlib==3.7.5",
        "accelerate==0.32.1",
        "omegaconf==2.3.0",
        "einops==0.8.0",
        "rank_bm25==0.2.2",
        "sentencepiece==0.2.0",
        "pyvi==0.1.1",
    ],
    author="Nam V. Nguyen, Dien X. Tran, Thanh T. Tran, Anh T. Hoang, Tai V. Duong, Di T. Le, Phuc-Lu Le",
    author_email="xuandienk4@gmail.com",
    description="SemViQA: A Semantic Question Answering System for Vietnamese Fact-Checking",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/DAVID-NGUYEN-S16/SemViQA",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Intended Audience :: Developers",
    ],
    python_requires=">=3.8",
    keywords="Vietnamese NLP, Fact-Checking, Question Answering, Machine Learning",
    project_urls={
        "Paper": "https://arxiv.org/abs/2503.00955",
        "Repository": "https://github.com/DAVID-NGUYEN-S16/SemViQA",
        "Competition Results": "https://codalab.lisn.upsaclay.fr/competitions/15497#results",
    },
)



import re

def preprocess_text(text: str) -> str:
    """
    Cleans and normalizes the input text by:
    - Removing punctuation marks (', ", ., ?, :, !).
    - Stripping leading and trailing whitespace.
    - Replacing multiple spaces with a single space.
    - Converting all characters to lowercase.

    Args:
        text (str): The input text.

    Returns:
        str: The cleaned and normalized text.
    """
    text = re.sub(r"['\",\.\?:\!]", "", text)
    text = text.strip()
    text = " ".join(text.split())
    return text.lower()

# def split_chill(t: str) -> list:
#     return re.split(r'(?<=\.)\s*', t)
 
# def split_sentence(paragraph: str) -> list:
#     context_list = sent_tokenize(paragraph)
#     updated_list = []
#     for context in context_list:
#         updated_list.extend(split_chill(context) if '. ' in context else [context])
#     return updated_list

def split_sentence(paragraph: str) -> list:
    """
    Splits a paragraph into sentences based on specific rules:
    - Recognizes sentence endings based on periods (".") followed by spaces.
    - Handles cases where a period is followed by a newline and an uppercase letter.
    - Ensures sentences have more than two words before adding them to the list.

    Args:
        paragraph (str): The input paragraph.

    Returns:
        list: A list of cleaned and preprocessed sentences.
    """
    sentences = []
    
    if paragraph.endswith("\n\n"):
        paragraph = paragraph[:-2]

    paragraph = paragraph.rstrip()
    start = 0
    paragraph_length = len(paragraph)

    while start < paragraph_length:
        sentence = ""
        initial_start = start

        for i in range(start, paragraph_length):
            if paragraph[i] == ".":
                if i + 2 < paragraph_length and paragraph[i + 1] == "\n":
                    if paragraph[i + 2].isalpha() and paragraph[i + 2].isupper():
                        break
                
                if i + 1 < paragraph_length and paragraph[i + 1] == " ":
                    sentence += paragraph[i]
                    start = i + 1
                    break

            sentence += paragraph[i]

            if i == paragraph_length - 1:
                start = paragraph_length
                break

        if start == paragraph_length:
            sentence += paragraph[start:]

        cleaned_sentence = preprocess_text(sentence.strip())
        if len(cleaned_sentence.split()) > 2:
            sentences.append(cleaned_sentence)

        if start == initial_start:
            print("Warning: No progress detected. Exiting loop.")
            break

    return sentences

def process_data(text: str) -> str:
    """
    Processes a paragraph by splitting it into sentences and returning them as a formatted string.

    Args:
        text (str): The input paragraph.

    Returns:
        str: A string where sentences are joined by ". ".
    """
    return ". ".join(split_sentence(text))


def load_data(data):
    data_old = {}

    for i in data.index:
        if data.id[i] not in data_old.keys():
            data_old[data.id[i]] = [
                {
                    'id': data.id[i],
                    'context': data.context[i],
                    'claim': data.claim[i]
                }
            ]
        else:
            data_old[data.id[i]].append(
                    {
                        'id': data.id[i],
                        'context': data.context[i],
                        'claim': data.claim[i]
                    }
                )
    return data_old



import torch
import torch.nn as nn
import torch.nn.functional as F
from torch.nn import CrossEntropyLoss

class RationaleRegularizationLoss(nn.Module):
    def __init__(self, lambda_sparse=0.01, lambda_continuity=0.01):
        """
        Implements rationale regularization as a PyTorch module.

        Args:
            lambda_sparse (float): Coefficient for sparsity loss.
            lambda_continuity (float): Coefficient for continuity loss.
        """
        super(RationaleRegularizationLoss, self).__init__()
        self.lambda_sparse = lambda_sparse
        self.lambda_continuity = lambda_continuity

    def forward(self, pt, mask=None):
        """
        Computes the regularization loss.

        Args:
            pt (torch.Tensor): Probability of each token being part of the rationale.
                               Shape: (batch_size, seq_len, 1)
            mask (torch.Tensor, optional): Mask for padding positions.
                                           Shape: (batch_size, seq_len)

        Returns:
            torch.Tensor: Total regularization loss (scalar).
        """
        pt = pt.squeeze(-1)  

        if mask is not None:
            pt = pt * mask  

        # Sparsity loss: L1 norm sum over sequence, then averaged over the batch
        sparsity_loss = pt.sum(dim=-1).mean()

        # Continuity loss: sum of squared differences between adjacent tokens
        continuity_loss = ((pt[:, 1:] - pt[:, :-1]) ** 2).mean()

        # Weighted sum of the two losses
        total_reg_loss = self.lambda_sparse * sparsity_loss + self.lambda_continuity * continuity_loss

        return total_reg_loss

class RTLoss(nn.Module):
    
    def __init__(self, device = 'cuda'):
        super(RTLoss, self).__init__()
        self.device = device
    
    def forward(self, pt: torch.Tensor, Tagging:  torch.Tensor):
        '''
        Tagging: list paragraphs contain value token. If token of the paragraphas is rationale will labeled 1 and other will be labeled 0 
        
        RT: 
                    p^r_t = sigmoid(w_2*RELU(W_1.h_t))
            
            With:
                    p^r_t constant
                    w_2 (d x 1)
                    W_1 (d x d)
                    h_t (1 x d)
                    
            This formular is compute to each token in paraphase. I has convert into each paraphase
            
                    p^r_t = sigmoid(w_2*RELU(W_1.h))
                    
                    With:
                            p^r (1 x n) with is number of paraphase
                            w_2 (d x 1)
                            W_1 (d x d)
                            h (n x d) 
                            
        '''
        
        Tagging = torch.tensor(Tagging, dtype=torch.float32).to(pt.device)
                
        total_loss = torch.tensor(0, dtype= torch.float32).to(pt.device)
        
        N = pt.shape[0]
                
        for i, text in enumerate(pt):
            T = len(Tagging[i])
            Lrti = -(1/T) * (Tagging[i]@torch.log(text) + (1.0 - Tagging[i]) @ torch.log(1.0 - text) )[0]
            total_loss += Lrti
            
        return total_loss/N


class comboLoss(nn.Module):
    def __init__(self, config):
        
        super(comboLoss, self).__init__()
        self.alpha = config.alpha
        self.beta = config.beta
        # self.BaseLoss = BaseLoss()
        self.RTLoss = RTLoss()
        self.reg_loss_fn = RationaleRegularizationLoss(lambda_sparse=0.01, lambda_continuity=0.01)
        self.config = config
        
    def forward(self, output: dict):
        attention_mask = output['attention_mask']
        start_logits = output['start_logits']
        end_logits = output['end_logits']
        
        start_positions = output['start_positions']
        end_positions = output['end_positions']
        
        Tagging = output['Tagging']
        pt = output['pt']

        loss_base = 0
        if start_positions is not None and end_positions is not None:
            # If we are on multi-GPU, split add a dimension
            if len(start_positions.size()) > 1:
                start_positions = start_positions.squeeze(-1)
            if len(end_positions.size()) > 1:
                end_positions = end_positions.squeeze(-1)
            # sometimes the start/end positions are outside our model inputs, we ignore these terms
            ignored_index = start_logits.size(1)
            start_positions = start_positions.clamp(0, ignored_index)
            end_positions = end_positions.clamp(0, ignored_index)

            loss_fct = CrossEntropyLoss(ignore_index=ignored_index)
            start_loss = loss_fct(start_logits, start_positions)
            end_loss = loss_fct(end_logits, end_positions)
            loss_base = (start_loss + end_loss) / 2
        retation_tagg_loss  = self.RTLoss(pt = pt, Tagging = Tagging)
        # retation_tagg_loss = nn.BCELoss()(pt, Tagging)
        # retation_tagg_loss = 0
        reg_loss = self.reg_loss_fn(pt, mask=attention_mask)
        total_loss = self.alpha*loss_base + reg_loss + self.beta*retation_tagg_loss 
        
        return total_loss




import logging
import os
import pandas as pd
import numpy as np
import torch
import gc
from sklearn.metrics import f1_score, cohen_kappa_score, matthews_corrcoef
from accelerate import Accelerator
from accelerate.logging import get_logger
from accelerate.utils import ProjectConfiguration, set_seed
from tqdm import tqdm
from transformers import default_data_collator
from torch.utils.data import DataLoader
import argparse
import time
from safetensors.torch import load_model
from transformers import AutoTokenizer

from .qatc_model import QATCConfig, QATCForQuestionAnswering
from .data_utils import load_data 

os.environ["WANDB__SERVICE_WAIT"] = "300"
os.environ["CUDA_VISIBLE_DEVICES"] = "0"

def count_parameters(model):
    total_params = sum(p.numel() for p in model.parameters())
    print(f"Total parameters: {total_params}")
    active_params = sum(p.numel() for p in model.parameters() if p.requires_grad)
    print(f"Trainable parameters: {active_params}")

def load_models(args):
    config = QATCConfig(
        model_name=args.model_name,
        freeze_text_encoder=args.freeze_text_encoder,
        alpha=args.alpha,
        beta=args.beta
    )
    
    model = QATCForQuestionAnswering(config)
    
    if args.is_load_weight:
        print("loading train weight")
        if "safetensors" in args.weight_model:
            load_model(model, f"{args.weight_model}")
        else:
            model.load_state_dict(torch.load(args.weight_model), strict=False)
    
    count_parameters(model)
    return model, config

def setting_optimizer(config, model):
    optimizer_cls = torch.optim.AdamW
    return optimizer_cls(model.parameters(), lr=config.learning_rate)

def main(args):
    logger = get_logger(__name__, log_level="INFO")

    logging_dir = os.path.join(args.output_dir, args.logging_dir)
    accelerator_project_config = ProjectConfiguration(
        project_dir=args.output_dir, logging_dir=logging_dir
    )

    accelerator = Accelerator(
        log_with=args.report_to,
        gradient_accumulation_steps=args.gradient_accumulation_steps,
        project_config=accelerator_project_config
    )

    logging.basicConfig(
        format="%(asctime)s - %(levelname)s - %(name)s - %(message)s",
        datefmt="%m/%d/%Y %H:%M:%S",
        level=logging.INFO,
    )
    logger.info(accelerator.state, main_process_only=False)

    if args.seed is not None:
        set_seed(args.seed)

    if accelerator.is_main_process and args.output_dir is not None:
        os.makedirs(args.output_dir, exist_ok=True)

    model, config = load_models(args)
    model = accelerator.prepare(model)
    tokenizer = AutoTokenizer.from_pretrained(args.model_name)

    optimizer = setting_optimizer(args, model)
    train_dataset, test_dataset = load_data(args, tokenizer)
    
    train_dataloader = DataLoader(
        train_dataset, shuffle=True, collate_fn=default_data_collator, batch_size=args.train_batch_size
    )
    
    eval_dataloader = DataLoader(
        test_dataset, collate_fn=default_data_collator, batch_size=args.train_batch_size
    )

    lr_scheduler = torch.optim.lr_scheduler.CyclicLR(optimizer, base_lr=args.learning_rate, max_lr=args.max_lr, cycle_momentum=False)

    if args.is_load_weight: 
        print("loading scheduler weight")
        lr_scheduler.load_state_dict(torch.load(args.weight_scheduler))

    optimizer, train_dataloader, lr_scheduler = accelerator.prepare(optimizer, train_dataloader, lr_scheduler)

    progress_bar = tqdm(desc="Steps", disable=not accelerator.is_local_main_process)
 
    best_acc = 0.0

    print("Starting training...")
    global_step = 1
    cnt = 0
    start_time = time.time()

    for epoch in range(args.num_train_epochs):
        model.train()
        train_loss = 0.0
        for step, batch in enumerate(train_dataloader):
            for key in batch:
                batch[key] = batch[key].to(accelerator.device)
            batch['Tagging'] = batch['Tagging'].to(torch.float32)
            output = model(
                input_ids=batch['input_ids'], 
                attention_mask=batch['attention_mask'], 
                start_positions=batch['start_positions'], 
                end_positions=batch['end_positions'],
                tagging_labels=batch['Tagging']
            )

            loss = output.loss
            accelerator.backward(loss)

            if accelerator.sync_gradients:
                accelerator.clip_grad_norm_(model.parameters(), args.max_grad_norm)
                
            if global_step % args.gradient_accumulation_steps == 0:
                optimizer.step()
                optimizer.zero_grad()
                lr_scheduler.step()

            avg_loss = accelerator.gather(loss.repeat(args.train_batch_size)).mean()
            train_loss += avg_loss.item() / args.gradient_accumulation_steps
            logs = {"step": f"{step}/{len(train_dataloader)}", "step_loss": loss.detach().item(), "lr": lr_scheduler.get_last_lr()[0]}
            progress_bar.set_postfix(logs)
            global_step += 1
            
            if global_step % args.max_iter == 0:
                train_loss = round(train_loss / args.max_iter, 4)

                print({
                        'global_step': global_step,
                        'Train loss': train_loss, 
                        "epoch": epoch,
                    }) 
                train_loss = 0.0

        train_loss /= len(train_dataloader)
        print(f"Epoch {epoch+1} - Train Loss: {train_loss}")

        model.eval()
        eval_loss = 0.0
        predictions, true_positions = [], []
        
        for step, batch in enumerate(eval_dataloader):
            with torch.no_grad():
                for key in batch:
                    batch[key] = batch[key].to(accelerator.device)
                batch['Tagging'] = batch['Tagging'].to(torch.float32)
                output = model(
                    input_ids=batch['input_ids'], 
                    attention_mask=batch['attention_mask'],
                    start_positions=batch['start_positions'],
                    end_positions=batch['end_positions'],
                    tagging_labels=batch['Tagging']
                )

                loss = output.loss
                eval_loss += loss.item()

                start_preds = torch.argmax(output.start_logits, axis=1).cpu().detach().tolist()
                end_preds = torch.argmax(output.end_logits, axis=1).cpu().detach().tolist()
                start_true = batch['start_positions'].flatten().cpu().tolist()
                end_true = batch['end_positions'].flatten().cpu().tolist()

                predictions.extend(list(zip(start_preds, end_preds)))
                true_positions.extend(list(zip(start_true, end_true)))
            
            progress_bar.update(1)

        eval_loss /= len(eval_dataloader)
        accuracy = np.mean([p == t for p, t in zip(predictions, true_positions)])

        print(f"Epoch {epoch+1} - Eval Loss: {eval_loss} - Accuracy: {accuracy}")

        if accuracy > best_acc:
            save_path = os.path.join(args.output_dir, f"best {args.name}")
            if not os.path.exists(save_path):
                os.makedirs(save_path)
            # accelerator.save_state(save_path)
            model.save_pretrained(save_path)
            torch.save(optimizer.state_dict(), os.path.join(save_path, "optimizer.bin"))
            torch.save(lr_scheduler.state_dict(), os.path.join(save_path, "scheduler.bin"))
            tokenizer.save_pretrained(save_path)
            config.save_pretrained(save_path)
            best_acc = accuracy
            print("Save model best acc at epoch", epoch)
            cnt = 0
        else:
            cnt += 1
        if cnt == args.patience:
            print(f"Early stopping at epoch {epoch}.")
            break

    print("Training completed in:", time.time() - start_time, "seconds.")

def parse_args():
    parser = argparse.ArgumentParser(description="Evaluate VNFASHIONDIFF")
    parser.add_argument("--name", type=str, default="QATC", help="Name of the model")
    parser.add_argument("--model_name", type=str, default="/kaggle/input/model-base", help="Path to the base model") 
    parser.add_argument("--output_dir", type=str, default="/kaggle/working/", help="Output directory")
    parser.add_argument("--seed", type=int, default=40, help="Random seed")
    parser.add_argument("--logging_dir", type=str, default="logs", help="Logging directory")
    parser.add_argument("--train_batch_size", type=int, default=12, help="Training batch size")
    parser.add_argument("--num_train_epochs", type=int, default=100, help="Number of training epochs")
    parser.add_argument("--num_epoch_eval", type=int, default=1, help="Number of epochs to evaluate")
    parser.add_argument("--gradient_accumulation_steps", type=int, default=1, help="Gradient accumulation steps")
    parser.add_argument("--use_8bit_adam", type=int, default=None, help="Use 8-bit Adam optimizer")
    parser.add_argument("--learning_rate", type=float, default=0.00001, help="Learning rate")
    parser.add_argument("--max_lr", type=float, default=0.00003, help="Maximum learning rate")
    parser.add_argument("--max_grad_norm", type=float, default=1.0, help="Maximum gradient norm")
    parser.add_argument("--report_to", type=str, default="wandb", help="Reporting destination (e.g., 'wandb')")
    
    # Data related arguments
    parser.add_argument("--train_data", type=str, default='train.csv', help="Path to training data")
    parser.add_argument("--eval_data", type=str, default='test.csv', help="Path to evaluation data")
    
    # Others
    parser.add_argument("--freeze_text_encoder", type=int, default=0, help="Whether to freeze the text encoder")
    parser.add_argument("--beta", type=float, default=0.1, help="Beta")
    parser.add_argument("--alpha", type=float, default=1, help="Alpha")
    parser.add_argument("--is_load_weight", type=int, default=0, help="Load weights from pre-trained model")
    parser.add_argument("--weight_model", type=str, default="/kaggle/input/weight-QACT/pytorch_model.bin", help="Path to model weights")
    parser.add_argument("--weight_optimizer", type=str, default="/kaggle/input/weight-QACT/optimizer.bin", help="Path to optimizer weights")
    parser.add_argument("--weight_scheduler", type=str, default="/kaggle/input/weight-QACT/scheduler.bin", help="Path to scheduler weights")
    parser.add_argument("--max_iter", type=int, default=100, help="Maximum number of iterations")
    parser.add_argument("--is_train", type=int, default=1, help="Set to True if training")
    parser.add_argument("--is_eval", type=int, default=1, help="Set to Eval")
    parser.add_argument("--patience", type=int, default=5, help="Patience for early stopping")
    args = parser.parse_args()
    return args
if __name__ == "__main__":
    args = parse_args()
    main(args)
